#!/usr/bin/env python 
"""
##########################################################################
LilyComp v1.0 

DESCRIPTION
    LilyComp is a graphical note entry program for use with the LilyPond
    music typesetting program (lilypond.org). This program was written to 
    speed up the creation of lead sheets.

AUTHOR
    Written by Blake Garretson, <blakeg@freeshell.org>
    Author's personal website: http://blakeg@freeshell.org

RESOURCES
    SourceForge: http://sourceforge.net/projects/lilycomp/
    Main Site: http://lilycomp.sourceforge.net/

COPYING
    Copyright (C) 2004 Blake T. Garretson. Free use of this software is
    granted under the terms of the GNU General Public License (GPL).

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
##########################################################################

"""
from Tkinter import *
import tkMessageBox, tkFileDialog

# These are the variables you can change
options = { 'width': 300, # canvas width
            'height': 320, # canvas height
            'staff_spacing': 30,
            'top_margin': 40,
            'staff_left': 20,
            'staff_right': 220,
            'ledger_left':85,
            'ledger_right':160,
            'upper_ledger_quant': 2,
            'lower_ledger_quant': 2,
            'note_size': 24,
            'note_x_space': 105,
            'note_x_line': 135,
            'note_font': "Helvetica 10 bold",
            'header_font': "Helvetica 10 bold",
            'widget_font': "Helvetica 10",
            'text_font': "Courier 10",
            'note_durations': ((1,'note_whole'), (2,'note_half'), (4,'note_quarter'), 
                (8,'note_eighth'), (16,'note_sixteenth'), (32,'note_thirtysecond'), 
                (64,'note_sixtyfourth')),
            'rest_durations': ((1,'rest_whole',30), (2,'rest_half',60), 
                (4,'rest_quarter',90), (8,'rest_eighth',145), 
                (16,'rest_sixteenth',190), (32,'rest_thirtysecond',245), 
                (64,'rest_sixtyfourth',245)),
            '#': "s", # defines sharp symbol for use with english.ly
            'b': "f", # defines flat symbol for use with english.ly
            'rest_xpos': 235,
            }

# The GUI is dynamic, so adding additional clefs here will add them to the GUI.
# The clefs' value indicates the pitch of the lowermost note displayed in c major.
# Changing the number of ledger lines screws this up, so update these if you add
# lower ledger lines.
clefs = { 'Treble': 'g',
          'Bass': 'b,,', 
        }

############################################################################
# The casual user probably shouldn't touch anything below here, unless they
# know Python and Tkinter.
############################################################################

images = {
    "bar": """R0lGODlhEgAdAID/AAAAAMDAwCH5BAEAAAEALAAAAAASAB0AQAI4hI95wc3mogxw2molzm7z
B1zKiIhf5H0pSJLm2ayZ/MJUaN9iO9YwPQFqcDYhinjkKXwnYwfpKAAAOzs=
""",
    "dot": """R0lGODlhEgAdAID/AAAAAMDAwCH5BAEAAAEALAAAAAASAB0AQAI5jI+pCuvWHpiQvhVvytjO
+FnaeHCkOaKaeoEhw26cSAaxc2N1uUO9/csxgiWa6FNBJpdM2clD+yUKADs=
""",
    "note_eighth": """R0lGODlhEgAdAJH/AP///wAAAMDAwAAAACH5BAEAAAIALAAAAAASAB0AQAI+lI+pGt3LonD0
QWhvClvP7oWgaDjkV3KeirCLWx6VKbczLOLrmfIUqcpoXMLXRlcbGZWSpGzWOkIxt9VPUwAA
Ow==
""",
    "note_half": """R0lGODlhDQAgAJH/AP///wAAAMDAwAAAACH5BAEAAAIALAAAAAANACAAQAIxlI+pFpuNgoyO
VnbxgULy/DnhMirlI3rftLHZC5/WJW9vrdH3Dppe3nEFZ8GRCrcoAAA7
""",
    "note_quarter": """R0lGODlhCwAdAJH/AP///wAAAMDAwAAAACH5BAEAAAIALAAAAAALAB0AQAIolI+ZocgSopTt
vHpbrnwvDoGeY00UZEZg2IluC2OvHKNmmRqpivNGAQA7
""",
    "note_sixteenth": """R0lGODlhEgAdAJH/AP///wAAAMDAwAAAACH5BAEAAAIALAAAAAASAB0AQAJClI+pGhe9EHTw
2Rvzywub2THcGJKSZQqbM1YbKCbT5UkfvaafvvNo2vi1ZjBGLhf6FW2UJUvGKYJw0iYNesMN
hZECADs=
""",
    "note_sixtyfourth": """R0lGODlhEgAlAJH/AP///wAAAMDAwAAAACH5BAEAAAIALAAAAAASACUAQAJYlI+pGhe90IPi
RdMmxrMv+oGXU1nVaJZU5yXiCG+xpM31idKvxMnRjfvpOKwUaXYUIn1IIAxivKSijNvuqLn2
HFrVKtfrGqm6r5Qb9DqnxRe0/eE6eUtIAQA7
""",
    "note_thirtysecond": """R0lGODlhEgAdAJH/AP///wAAAMDAwAAAACH5BAEAAAIALAAAAAASAB0AQAJFlI+pGhe90HvO
zEsTjDkyC22LKHTe+aHapZYdyWzTKTc1Bod4Dkut1POwWrog0GEEvWjIEu2VtGWiQNEu5cKl
dr2baVEAADs=
""",
    "note_whole": """R0lGODlhFgAOAIAAAMDAwAAAACH5BAEAAAAALAAAAAAWAA4AQAIhhI+pGh3pForGPYmz3pNW
r1kXR2LimFldk4KAy3hwSXMFADs=
""",
    "rest_eighth": """R0lGODlhOgAaALMAAP///+/v79/f38/Pz7u7u6qqqpqamoqKinZ2dmZmZlVVVUVFRTExMSEh
IREREQAAACwAAAAAOgAaAEAEdxDISau9OOvNZ1hP+CjGU3RouhVPoL5wLMOI6AiTQcy8NjyD
ntBycAyPyCRFFDIAAoyG8phYTIeLxFXYQGy/4DD40GDsxCoH8yAZ3NCaBNNQfHjhm8GhgCu5
8CkObIAoRYQoAQ9OhxwJDIwcAg9nkBkFi5WZmhURADs=
""",
    "rest_half": """R0lGODlhOgAQALP/AP////r6+vHx8czMzMfHx7W1tZmZmYaGhoWFhSIiIhQUFAAAAAAAAAAA
AAAAAAAAACwAAAAAOgAQAEAEQxDISau9OOvN5/pg+HVkSYnoYq5s675wbKWi3B10ri+H7f/A
oHBY2qmIHSOSQjA4n86ETgGFFoCCgXbL7Xq/y7DYFwEAOw==
""",
    "rest_quarter": """R0lGODlhOgAlALP/AP///+/v79/f38/Pz7u7u6qqqpqamoqKimZmZlVVVUVFRTExMSEhIRER
EQAAAAAAACwAAAAAOgAlAEAEnxDISau9OOvN73GO0o2ktoBiqa4I6LrDKpPHMt+jwgR4rzGu
Bc9HnAwciCKRAGIon9CoFJpwFKY9oEuALR1dh+7sYxOvBgRCYJEyc15DSczNaYXp+Lye85nv
OQJIfx0fDVyDGS0gd4gVBS9JjRQfIIaSk3CXFV9tmhQnh56YoZIBCi4NJ4yNLwsGBAcMTpJV
DmWiEoqkuAgNfrgSccBSEQA7
""",
    "rest_sixteenth": """R0lGODlhOgAjALP/AP///+/v79/f38/Pz7u7u6qqqpqamoqKinZ2dmZmZlVVVUVFRTExMSEh
IREREQAAACwAAAAAOgAjAEAErxDISau9OOvN7WhP6CDHE3RoOjlhWACFw6h0PTGIre98bzGt
BgVx8hknB8dxCVAomEfHAUqt7git0ABgeCSsPSm4k8gKJIGHYYxCBA+DB4FtKzzOdFo3X0MI
+YCBghwBCQwKW4MZcVlzXH+KEyAtBmUPL5EVBJsSCguZGnGJkQELDw5rN1+gWQ9TMCagAG4t
CZavshIEeEm5GGq+FggzwRQCl8UUTskTWKPJA8/MVBEAOw==
""",
    "rest_sixtyfourth": """R0lGODlhIQA4ALP/AP///+/v79/f38/Pz7u7u6qqqpqamoqKinZ2dmZmZlVVVUVFRTExMSEh
IREREQAAACwAAAAAIQA4AEAE/xDISau9OEvS3muI0ByadnheAwTMU5RZ5zlT8QRwNgjVoeZA
wEIRBDpIxaQB9UhMCM7kJECwPAxSzeAxyEoCCxTjcKB5HUyswbHwxhDuuFwi6M4pCqZz0MLd
J2VuW2ITCD+CYQ8OCVBtfxMLUVmDKF0ncI8SR255KFUrLnJlHmMEXJkANjyZS6gAPq6xshkI
Hgt+d522Xw4vbqYokYoMrlufmQpEbmsfdqkPq1knTF8gcQFoKQgOaLGbqIGCHQ7HAVeCTA+f
CcRxLSgHicfXBH7JxdBuLB6SAAz9UtIhWYJLmp4FaJBkCpfJHBZUCNplEhAK1T1Uppw9GqBx
lsckEQEAADs=
""",
    "rest_thirtysecond": """R0lGODlhGgA4ALMAAP///+/v79/f38/Pz7u7u6qqqpqamoqKinZ2dmZmZlVVVUVFRTExMSEh
IREREQAAACwAAAAAGgA4AEAEyxDISau9OOsb2vvN8nibRHwoISlMCQTMd1AM4maPcV+nsP/A
SeLjGFCMwQmiEVStFLcD6pFYeGZJgODhzHq/4PBOimq9amAHtjTwOLqBR2EnQh0WDu63d5s+
sFIBXwcOYAhmXgpQXwNIYo+QkZKTPwMEglkFU4sAAwtjm1oxXS4BCXk1AopgJ45ZDAk7HkQT
Bg+YbH5GQ2ulQx8MBgaFaTo7Az4USzsKU0hxczd5KHO2nz8Hs8EDD65JC7FfmsleDTZftmFq
YNmU7pMRADs=
""",
    "rest_whole": """R0lGODlhOgAMALP/AP////r6+vHx8czMzMfHx7W1tZmZmYaGhoWFhSIiIhQUFAAAAAAAAAAA
AAAAAAAAACwAAAAAOgAMAEAEPRDISau9OOvN5/pg+HWkVhhoiiqimKgqUZJDbd94rgtzT7Ut
n3BILBqPlgNwyQwdkJzmCCqULqhVKXZ7jAAAOw==
""",
    "slurbegin": """R0lGODlhEgAdAID/AAAAAMDAwCH5BAEAAAEALAAAAAASAB0AQAIyjI+pB73bwJvBUWVplJfl
joGM+GnlFIlVqp6dC3JvK7/1xdowtvO3+QPSgqgehKhRFAAAO8lD+yUKADs=
""",
    "slurend": """R0lGODlhEgAdAID/AAAAAMDAwCH5BAEAAAEALAAAAAASAB0AQAIzjI+pCr0fGoDI0Tovk9pw
mnVeKI7lSGqful4W26XiW9KsDNq5DrUu//D1gAthkMio4Q4FADtD+yUKADs=
""",
    "tie": """R0lGODlhGAAdAID/AAAAAMDAwCH5BAEAAAEALAAAAAAYAB0AQAIujI+py+0Po5wBAGRzVpb6
D4aOpj3XNZKlyLbuC8cJKaEqzdj5XXVVpJIJh8RhAQA7ZEKp5YjWnJmYE53yaSUqCgA7
""",
}

keys = { "#": ('f','c','g','d','a','e','b'), #order
         "b": ('b','e','a','d','g','c','f')
        }

# define the lilypond note order (without #'s or b's)
scale = [   
        "c,,,","d,,,","e,,,","f,,,","g,,,","a,,,","b,,,",
        "c,,","d,,","e,,","f,,","g,,","a,,","b,,",
        "c,","d,","e,","f,","g,","a,","b,",
        "c","d","e","f","g","a","b",
        "c'","d'","e'","f'","g'","a'","b'",
        "c''","d''","e''","f''","g''","a''","b''",
        "c'''","d'''","e'''","f'''","g'''","a'''","b'''",
        "c''''","d''''","e''''","f''''","g''''","a''''","b''''",
        ]

options['half_note_size'] = options['note_size']/2.0

class LilyComp(Frame):
    def __init__(self, master=None):
        Frame.__init__(self, master, border=0)
        self.pack(fill=BOTH,expand=1,side=TOP)
        # Initialize Variables
        self.keytype = StringVar()
        self.keytype.set("#")
        self.keynumber = IntVar()
        self.keynumber.set(0)
        self.clef = StringVar()
        self.clef.set("Treble")
        self.duration = IntVar()
        self.duration.set(1)
        self.load_pics()
        # Create Controls
        self.create_controls()
        # Initialize Canvas
        self.draw_canvas()
        # Create Text Window
        self.text = Text(self, height=5, width=1, # width will expand to canvas size
                        font=options['text_font'],wrap=WORD)
        self.text.pack(side=TOP, fill=BOTH, expand=1)
    def load_pics(self):
        self.pics = {}
        for i in images.keys():
            self.pics[i] = PhotoImage(data=images[i])
    def create_controls(self):
        self.controlsframe = Frame(self)
        self.controlsframe.pack(side=TOP, fill=BOTH, expand=1)
        self.clefkeysframe = Frame(self.controlsframe)
        self.clefkeysframe.pack(side=LEFT, fill=BOTH, expand=1)
        # Clefs
        self.clefsframe = Frame(self.clefkeysframe, bd=1, relief="sunken")
        self.clefsframe.pack(side=TOP, fill=BOTH, expand=1)
        Label(self.clefsframe, text="Clef", font=options['header_font']).pack(side=TOP)
        clefnames = clefs.keys()
        for name in clefnames:
            Radiobutton(self.clefsframe, text=name, command=self.update_clef,
                variable=self.clef, value=name).pack(side=TOP,anchor=W)
        # Keys
        self.keyframe = Frame(self.clefkeysframe, bd=1, relief="sunken")
        self.keyframe.pack(side=TOP, fill=BOTH, expand=1)
        Label(self.keyframe, text="Key", font=options['header_font']).pack(side=TOP)
        for keytype in ('#','b'):
            Radiobutton(self.keyframe, text=keytype, command=self.update_key,
                variable=self.keytype, value=keytype, 
                font=options['widget_font']).pack(side=TOP)
        Label(self.keyframe, textvariable=self.keynumber, 
            font=options['widget_font']).pack(side=TOP)
        Scale(self.keyframe, command=self.update_key, from_=0, to=7, 
            variable=self.keynumber, orient=VERTICAL, showvalue=0, 
            font=options['widget_font']).pack(side=TOP)
        # Durations
        self.durationframe = Frame(self.controlsframe, bd=1, relief="sunken")
        self.durationframe.pack(side=LEFT, fill=BOTH, expand=1)
        Label(self.durationframe, text="Note", 
            font=options['header_font']).pack(side=TOP)
        for num, img in options['note_durations']:
            Radiobutton(self.durationframe, text=str(num), image=self.pics[img], 
                command=self.update_duration, variable=self.duration, value=num, 
                indicatoron=0,font=options['widget_font']).pack(fill=BOTH,
                expand=1,side=TOP,anchor=W)
        # Create Canvas
        self.canvasframe = Frame(self.controlsframe, bd=0)
        self.canvasframe.pack(side=LEFT, fill=BOTH, expand=1)
        self.canvas = Canvas(self.canvasframe, width=options['width'],
                            height=options['height'], bg='white', 
                            bd=1, relief="sunken")
        self.canvas.pack(side=TOP, fill=BOTH, expand=1)
        self.canvas.bind("<Button-1>", self.canvas_click)
        self.canvasbuttonsframe = Frame(self.canvasframe, bd=0)
        self.canvasbuttonsframe.pack(side=TOP, fill=BOTH, expand=1)
        Button(self.canvasbuttonsframe,image=self.pics['dot'],
            border=3, command=self.dot).pack(side=LEFT,fill=BOTH,expand=1)
        Button(self.canvasbuttonsframe,image=self.pics['tie'],
            border=3, command=self.tie).pack(side=LEFT,fill=BOTH,expand=1)
        Button(self.canvasbuttonsframe,image=self.pics['slurbegin'],
            border=3, command=lambda a=self.slur, b=1:a(b)).pack(side=LEFT,fill=BOTH,expand=1)
        Button(self.canvasbuttonsframe,image=self.pics['slurend'],
            border=3, command=lambda a=self.slur, b=0:a(b)).pack(side=LEFT,fill=BOTH,expand=1)
        Button(self.canvasbuttonsframe,image=self.pics['bar'],
            border=3, command=self.bar).pack(side=LEFT,fill=BOTH,expand=1)
    def canvas_click(self,event):
        x = self.canvas.canvasx(event.x)
        y = self.canvas.canvasx(event.y)
        item = self.canvas.find_closest(x, y)[0]
        if self.note_ids.has_key(item):
            self.text.insert(INSERT,"%s%s " % (self.note_ids[item],
                self.duration.get()))
        elif self.rest_ids.has_key(item):
            self.text.insert(INSERT,"r%s " % self.rest_ids[item])
    def bar(self, event=None):
        self.text.insert(INSERT,"| ")
    def slur(self, begin=0):
        i = self.text.index(INSERT)
        dot_loc = i.find(".")
        i2 = i[:dot_loc+1]+str(int(i[dot_loc+1:])-1)
        if begin:
            t = "("
        else:
            t = ")"
        self.text.insert(i2,t)
    def tie(self):
        self.text.insert(INSERT,"~ ")
    def dot(self):
        i = self.text.index(INSERT)
        dot_loc = i.find(".")
        i2 = i[:dot_loc+1]+str(int(i[dot_loc+1:])-1)
        self.text.insert(i2,".")
    def update_key(self, event=None):
        self.draw_canvas()
    def update_clef(self, event=None):
        self.draw_canvas()
    def update_duration(self, event=None):
        pass
    def draw_canvas(self):
        self.canvas.delete(self.canvas.find_all())
        self.note_ids = {}# id: "c'"
        self.rest_ids = {}# id: 1
        y = options['top_margin']
        y = self.draw_ledger(y,options['upper_ledger_quant'])
        y = self.draw_staff(y)
        y = self.draw_ledger(y,options['lower_ledger_quant'])
        number_of_notes = 11 + 2*options['lower_ledger_quant'] + 2*options['upper_ledger_quant']
        current_note = clefs[self.clef.get()]
        half_staff_spacing = options['staff_spacing']/2.0
        y -= half_staff_spacing
        space = 1
        for i in range(number_of_notes):
            if space:
                id1, id2 = self.draw_note(options['note_x_space'], y, 
                    self.get_note_in_key(current_note[0],0), color="yellow")
                space = 0
            else:
                id1, id2 = self.draw_note(options['note_x_line'], y, 
                    self.get_note_in_key(current_note[0],0), color="cyan")
                space = 1
            note_to_save = self.get_note_in_key(current_note[0],1)+current_note[1:]
            self.note_ids[id1] = note_to_save
            self.note_ids[id2] = note_to_save
            current_note = self.get_note(current_note, 1)
            y -= half_staff_spacing
        # draw rests
        for num, img, ypos in options['rest_durations']:
            if img == 'rest_sixtyfourth':
                xpos = options['rest_xpos']+26
            else:
                xpos = options['rest_xpos']
            id = self.canvas.create_image(xpos, ypos, anchor=NW, 
                image=self.pics[img])
            self.rest_ids[id] = num
    def get_note(self, note, inc): # inc can be pos or neg
        index = scale.index(note)
        return scale[index+inc]
    def draw_ledger(self, starting_y, quantity):
        current_y = starting_y
        for x in range(quantity):
            self.canvas.create_line(options['ledger_left'], current_y,
                                    options['ledger_right'], current_y,
                                    width=3)
            current_y = current_y + options['staff_spacing']
        return current_y
    def draw_note(self, x, y, text, color='red'):
        id1 = self.canvas.create_oval(x-options['half_note_size'],
            y-options['half_note_size'], x+options['half_note_size'],
            y+options['half_note_size'], fill=color)
        id2 = self.canvas.create_text(x, y, text=text, 
            font=options['note_font'], fill='black')
        return id1, id2
    def draw_staff(self, starting_y):
        current_y = starting_y
        for x in range(5):
            self.canvas.create_line(options['staff_left'], current_y,
                options['staff_right'], current_y, width=3)
            current_y = current_y + options['staff_spacing']
        return current_y
    def get_note_in_key(self, note, lilypond_notation=1): 
        """
        - note should just be a single character, e.g. "a" or "g"
        - 'lilypond_notation' determines whether you want C# or cs.
        """
        if note in keys[self.keytype.get()][:self.keynumber.get()]:
            if lilypond_notation:
                note = note + options[self.keytype.get()]
            else:
                note = note.upper() + self.keytype.get()
        else:
            if not lilypond_notation:
                note = note.upper()
        return note
    def about(self):
        tkMessageBox.showinfo("About","LilyComp 1.0.2\nCopyright 2004 Blake Garretson\n"
            "http://lilycomp.sourceforge.net/\n"
            "\nLilyComp comes with ABSOLUTELY NO WARRANTY. This is free"
            " software, and you are welcome to redistribute it under "
            "certain conditions. Read the LICENSE file which came with this"
            " program, and read the notice in lilycomp.py for details.")
    def help(self):
        tkMessageBox.showinfo("Help","See the lilycomp.html file.")
    def save(self):
        filename=tkFileDialog.asksaveasfilename(filetypes=[("Lilypond Files", "*.ly"),
                ("Text Files", "*.txt"),("All Files","*.*")])
        if filename:
            f = open(filename,'w')
            f.write(self.text.get(1.0, END))
            f.close()
    def copy_all(self):
        self.clipboard_clear()
        self.clipboard_append(self.text.get(1.0, END))

if __name__=='__main__':
    root = Tk()
    root.title("LilyComp")
    AppInstance = LilyComp(root)
    menubar = Menu(root)
    # create a pulldown menu, and add it to the menu bar
    filemenu = Menu(menubar, tearoff=0)
    filemenu.add_command(label="Save As...", command=AppInstance.save)
    filemenu.add_command(label="Copy All", command=AppInstance.copy_all)
    filemenu.add_separator()
    filemenu.add_command(label="Exit", command=root.destroy)
    menubar.add_cascade(label="File", menu=filemenu)
    #
    helpmenu = Menu(menubar, tearoff=0)
    helpmenu.add_command(label="Help", command=AppInstance.help)
    helpmenu.add_command(label="About", command=AppInstance.about)
    menubar.add_cascade(label="Help", menu=helpmenu)
    #
    root.config(menu=menubar)
    root.resizable(0,0)
    root.mainloop()

